"""
=========
CapStyle
=========

The `matplotlib._enums.CapStyle` controls how Matplotlib draws the corners
where two different line segments meet. For more details, see the
`~matplotlib._enums.CapStyle` docs.
"""

import matplotlib.pyplot as plt
from matplotlib._enums import CapStyle

CapStyle.demo()
plt.show()
